# coding: utf-8

# flake8: noqa
"""
    Warg Registry API

    [warg](https://warg.io/) is an open source protocol for WebAssembly component registries. 

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from warg_openapi.models.bundle_failure_error import BundleFailureError
from warg_openapi.models.checkpoint import Checkpoint
from warg_openapi.models.checkpoint_verification_response import CheckpointVerificationResponse
from warg_openapi.models.content_sources_response import ContentSourcesResponse
from warg_openapi.models.envelope_body import EnvelopeBody
from warg_openapi.models.error import Error
from warg_openapi.models.fetch_logs404_response import FetchLogs404Response
from warg_openapi.models.fetch_logs_id_not_found_error import FetchLogsIDNotFoundError
from warg_openapi.models.fetch_logs_log_length_not_found_error import FetchLogsLogLengthNotFoundError
from warg_openapi.models.fetch_logs_request import FetchLogsRequest
from warg_openapi.models.fetch_logs_response import FetchLogsResponse
from warg_openapi.models.fetch_names404_response import FetchNames404Response
from warg_openapi.models.fetch_package_names_request import FetchPackageNamesRequest
from warg_openapi.models.fetch_package_names_response import FetchPackageNamesResponse
from warg_openapi.models.fetch_warning import FetchWarning
from warg_openapi.models.get_content_sources404_response import GetContentSources404Response
from warg_openapi.models.get_package_record404_response import GetPackageRecord404Response
from warg_openapi.models.http_get import HttpGet
from warg_openapi.models.http_upload import HttpUpload
from warg_openapi.models.http_upload_headers import HttpUploadHeaders
from warg_openapi.models.incorrect_proof_error import IncorrectProofError
from warg_openapi.models.ledger_sources_response import LedgerSourcesResponse
from warg_openapi.models.ledger_sources_response_sources_inner import LedgerSourcesResponseSourcesInner
from warg_openapi.models.missing_content import MissingContent
from warg_openapi.models.package_not_included_error import PackageNotIncludedError
from warg_openapi.models.package_record import PackageRecord
from warg_openapi.models.processing_record import ProcessingRecord
from warg_openapi.models.prove_consistency404_response import ProveConsistency404Response
from warg_openapi.models.prove_consistency_request import ProveConsistencyRequest
from warg_openapi.models.prove_consistency_response import ProveConsistencyResponse
from warg_openapi.models.prove_inclusion404_response import ProveInclusion404Response
from warg_openapi.models.prove_inclusion422_response import ProveInclusion422Response
from warg_openapi.models.prove_inclusion_request import ProveInclusionRequest
from warg_openapi.models.prove_inclusion_response import ProveInclusionResponse
from warg_openapi.models.publish_package_record404_response import PublishPackageRecord404Response
from warg_openapi.models.publish_package_record409_response import PublishPackageRecord409Response
from warg_openapi.models.publish_package_record_request import PublishPackageRecordRequest
from warg_openapi.models.published_record import PublishedRecord
from warg_openapi.models.published_record_envelope import PublishedRecordEnvelope
from warg_openapi.models.rejected_record import RejectedRecord
from warg_openapi.models.signature import Signature
from warg_openapi.models.signed_checkpoint import SignedCheckpoint
from warg_openapi.models.sourcing_record import SourcingRecord
from warg_openapi.models.timestamped_checkpoint import TimestampedCheckpoint



# coding: utf-8

"""
    Warg Registry API

    [warg](https://warg.io/) is an open source protocol for WebAssembly component registries. 

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic import BaseModel, Field, StrictInt, StrictStr, constr, validator

class IncorrectProofError(BaseModel):
    """
    IncorrectProofError
    """
    status: StrictInt = Field(default=..., description="The HTTP status code for the error.")
    reason: StrictStr = Field(default=..., description="The reason why the bundle could not be generated.")
    root: constr(strict=True) = Field(default=..., description="Represents a supported hash.")
    found: constr(strict=True) = Field(default=..., description="Represents a supported hash.")
    __properties = ["status", "reason", "root", "found"]

    @validator('reason')
    def reason_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('incorrectProof',):
            raise ValueError("must be one of enum values ('incorrectProof')")
        return value

    @validator('root')
    def root_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-z0-9-]+:[a-f0-9]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-z0-9-]+:[a-f0-9]+$/")
        return value

    @validator('found')
    def found_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-z0-9-]+:[a-f0-9]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-z0-9-]+:[a-f0-9]+$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> IncorrectProofError:
        """Create an instance of IncorrectProofError from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> IncorrectProofError:
        """Create an instance of IncorrectProofError from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return IncorrectProofError.parse_obj(obj)

        _obj = IncorrectProofError.parse_obj({
            "status": obj.get("status"),
            "reason": obj.get("reason"),
            "root": obj.get("root"),
            "found": obj.get("found")
        })
        return _obj






"""
Cryptography support for interacting with warg-server.

Warg uses ECC secp256r1 for signing content uploads.

Implementation relies on cryptography package (using OpenSSL internally).

"""

import base64
import hashlib

from cryptography import exceptions
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives.serialization import (
    Encoding,
    NoEncryption,
    PrivateFormat,
    PublicFormat,
)


class PrivateKey:

    CURVES = {
        'ecdsa-p256': ec.SECP256R1(),
    }

    @classmethod
    def generate(cls) -> ec.EllipticCurvePrivateKey:
        """Generate a new private key"""
        return cls(ec.generate_private_key(curve=ec.SECP256R1()))

    @classmethod
    def load(cls, text: str) -> ec.EllipticCurvePrivateKey:
        """
        Decode a key in `<algo>:<base64>` format.
        """
        if ':' not in text:
            raise ValueError('required format: <algo>:<base64>')

        algo, key_b64 = text.split(':', 1)

        curve = cls.CURVES.get(algo)
        if not curve:
            raise ValueError('algorithm not supported: {}'.format(algo))

        key_bytes = base64.b64decode(key_b64)
        # todo: compare bytes length with curve key length

        key_int = int.from_bytes(key_bytes, byteorder='big')
        key = ec.derive_private_key(key_int, curve=curve)
        return cls(key)

    def __init__(self, key: ec.EllipticCurvePrivateKey):
        self.key = key

    def sign(self, data):
        """Generate signature"""
        return self.key.sign(data, signature_algorithm=ec.ECDSA(hashes.SHA256()))

    def public_key(self):
        return PublicKey(self.key.public_key())

    def pem_pkcs8(self):
        res = self.key.private_bytes(Encoding.PEM, PrivateFormat.PKCS8, NoEncryption())
        return res.decode('ascii')

    def pem_openssl(self):
        res = self.key.private_bytes(Encoding.PEM, PrivateFormat.TraditionalOpenSSL, NoEncryption())
        return res.decode('ascii')

    def der_pkcs8(self):
        return self.key.private_bytes(Encoding.DER, PrivateFormat.PKCS8, NoEncryption())

    def der_openssl(self):
        return self.key.private_bytes(Encoding.DER, PrivateFormat.TraditionalOpenSSL, NoEncryption())

    def canonical(self):
        """Convert to `<algo>:<base64>` format."""
        key_int = self.key.private_numbers().private_value
        key_bytes = key_int.to_bytes(length=32, byteorder='big')
        key_b64 = base64.b64encode(key_bytes).decode('ascii')
        return "{}:{}".format('ecdsa-p256', key_b64)


class PublicKey:

    def __init__(self, key: ec.EllipticCurvePublicKey):
        self.key = key

    def verify(self, signature, data):
        """Verify signature"""
        try:
            self.key.verify(signature=signature, data=data, signature_algorithm=ec.ECDSA(hashes.SHA256()))
        except exceptions.InvalidSignature:
            return False
        else:
            return True

    @property
    def public_numbers(self):
        return self.key.public_numbers()

    def pem(self):
        res = self.key.public_bytes(Encoding.PEM, PublicFormat.SubjectPublicKeyInfo)
        return res.decode('ascii')

    def der(self):
        return self.key.public_bytes(Encoding.DER, PublicFormat.SubjectPublicKeyInfo)

    def ssh(self):
        res = self.key.public_bytes(Encoding.OpenSSH, PublicFormat.OpenSSH)
        return res.decode('ascii')

    def x962_compressed_point(self):
        return self.key.public_bytes(Encoding.X962, PublicFormat.CompressedPoint)

    def x962_uncompressed_point(self):
        return self.key.public_bytes(Encoding.X962, PublicFormat.UncompressedPoint)

    def canonical(self):
        """Convert to `<algo>:<base64>` format."""
        key_bytes = self.x962_compressed_point()
        key_b64 = base64.b64encode(key_bytes).decode('ascii')
        return "{}:{}".format('ecdsa-p256', key_b64)

    def fingerprint(self):
        """Used as Key ID"""
        return "sha256:{}".format(hashlib.sha256(self.canonical().encode('ascii')).hexdigest())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: proto/warg/protocol/warg.proto
# Protobuf Python Version: 6.32.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    32,
    1,
    '',
    'proto/warg/protocol/warg.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1eproto/warg/protocol/warg.proto\x12\rwarg.protocol\x1a\x1fgoogle/protobuf/timestamp.proto\"?\n\x08\x45nvelope\x12\x10\n\x08\x63ontents\x18\x01 \x01(\x0c\x12\x0e\n\x06key_id\x18\x02 \x01(\t\x12\x11\n\tsignature\x18\x03 \x01(\t\"\x96\x01\n\x0eOperatorRecord\x12\x11\n\x04prev\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0f\n\x07version\x18\x02 \x01(\r\x12(\n\x04time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\x07\x65ntries\x18\x04 \x03(\x0b\x32\x1c.warg.protocol.OperatorEntryB\x07\n\x05_prev\"\xc2\x02\n\rOperatorEntry\x12+\n\x04init\x18\x01 \x01(\x0b\x32\x1b.warg.protocol.OperatorInitH\x00\x12\x36\n\ngrant_flat\x18\x02 \x01(\x0b\x32 .warg.protocol.OperatorGrantFlatH\x00\x12\x38\n\x0brevoke_flat\x18\x03 \x01(\x0b\x32!.warg.protocol.OperatorRevokeFlatH\x00\x12\x42\n\x10\x64\x65\x66ine_namespace\x18\x04 \x01(\x0b\x32&.warg.protocol.OperatorDefineNamespaceH\x00\x12\x42\n\x10import_namespace\x18\x05 \x01(\x0b\x32&.warg.protocol.OperatorImportNamespaceH\x00\x42\n\n\x08\x63ontents\"3\n\x0cOperatorInit\x12\x16\n\x0ehash_algorithm\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\"X\n\x11OperatorGrantFlat\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x36\n\x0bpermissions\x18\x02 \x03(\x0e\x32!.warg.protocol.OperatorPermission\"\\\n\x12OperatorRevokeFlat\x12\x0e\n\x06key_id\x18\x01 \x01(\t\x12\x36\n\x0bpermissions\x18\x02 \x03(\x0e\x32!.warg.protocol.OperatorPermission\",\n\x17OperatorDefineNamespace\x12\x11\n\tnamespace\x18\x01 \x01(\t\">\n\x17OperatorImportNamespace\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\x10\n\x08registry\x18\x02 \x01(\t\"\x94\x01\n\rPackageRecord\x12\x11\n\x04prev\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0f\n\x07version\x18\x02 \x01(\r\x12(\n\x04time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x07\x65ntries\x18\x04 \x03(\x0b\x32\x1b.warg.protocol.PackageEntryB\x07\n\x05_prev\"\x94\x02\n\x0cPackageEntry\x12*\n\x04init\x18\x01 \x01(\x0b\x32\x1a.warg.protocol.PackageInitH\x00\x12\x35\n\ngrant_flat\x18\x02 \x01(\x0b\x32\x1f.warg.protocol.PackageGrantFlatH\x00\x12\x37\n\x0brevoke_flat\x18\x03 \x01(\x0b\x32 .warg.protocol.PackageRevokeFlatH\x00\x12\x30\n\x07release\x18\x04 \x01(\x0b\x32\x1d.warg.protocol.PackageReleaseH\x00\x12*\n\x04yank\x18\x05 \x01(\x0b\x32\x1a.warg.protocol.PackageYankH\x00\x42\n\n\x08\x63ontents\"2\n\x0bPackageInit\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x16\n\x0ehash_algorithm\x18\x02 \x01(\t\"V\n\x10PackageGrantFlat\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x35\n\x0bpermissions\x18\x02 \x03(\x0e\x32 .warg.protocol.PackagePermission\"Z\n\x11PackageRevokeFlat\x12\x0e\n\x06key_id\x18\x01 \x01(\t\x12\x35\n\x0bpermissions\x18\x02 \x03(\x0e\x32 .warg.protocol.PackagePermission\"7\n\x0ePackageRelease\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x14\n\x0c\x63ontent_hash\x18\x02 \x01(\t\"\x1e\n\x0bPackageYank\x12\x0f\n\x07version\x18\x01 \x01(\t*\xad\x01\n\x12OperatorPermission\x12#\n\x1fOPERATOR_PERMISSION_UNSPECIFIED\x10\x00\x12\x1e\n\x1aOPERATOR_PERMISSION_COMMIT\x10\x01\x12(\n$OPERATOR_PERMISSION_DEFINE_NAMESPACE\x10\x02\x12(\n$OPERATOR_PERMISSION_IMPORT_NAMESPACE\x10\x03*t\n\x11PackagePermission\x12\"\n\x1ePACKAGE_PERMISSION_UNSPECIFIED\x10\x00\x12\x1e\n\x1aPACKAGE_PERMISSION_RELEASE\x10\x01\x12\x1b\n\x17PACKAGE_PERMISSION_YANK\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.warg.protocol.warg_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_OPERATORPERMISSION']._serialized_start=1724
  _globals['_OPERATORPERMISSION']._serialized_end=1897
  _globals['_PACKAGEPERMISSION']._serialized_start=1899
  _globals['_PACKAGEPERMISSION']._serialized_end=2015
  _globals['_ENVELOPE']._serialized_start=82
  _globals['_ENVELOPE']._serialized_end=145
  _globals['_OPERATORRECORD']._serialized_start=148
  _globals['_OPERATORRECORD']._serialized_end=298
  _globals['_OPERATORENTRY']._serialized_start=301
  _globals['_OPERATORENTRY']._serialized_end=623
  _globals['_OPERATORINIT']._serialized_start=625
  _globals['_OPERATORINIT']._serialized_end=676
  _globals['_OPERATORGRANTFLAT']._serialized_start=678
  _globals['_OPERATORGRANTFLAT']._serialized_end=766
  _globals['_OPERATORREVOKEFLAT']._serialized_start=768
  _globals['_OPERATORREVOKEFLAT']._serialized_end=860
  _globals['_OPERATORDEFINENAMESPACE']._serialized_start=862
  _globals['_OPERATORDEFINENAMESPACE']._serialized_end=906
  _globals['_OPERATORIMPORTNAMESPACE']._serialized_start=908
  _globals['_OPERATORIMPORTNAMESPACE']._serialized_end=970
  _globals['_PACKAGERECORD']._serialized_start=973
  _globals['_PACKAGERECORD']._serialized_end=1121
  _globals['_PACKAGEENTRY']._serialized_start=1124
  _globals['_PACKAGEENTRY']._serialized_end=1400
  _globals['_PACKAGEINIT']._serialized_start=1402
  _globals['_PACKAGEINIT']._serialized_end=1452
  _globals['_PACKAGEGRANTFLAT']._serialized_start=1454
  _globals['_PACKAGEGRANTFLAT']._serialized_end=1540
  _globals['_PACKAGEREVOKEFLAT']._serialized_start=1542
  _globals['_PACKAGEREVOKEFLAT']._serialized_end=1632
  _globals['_PACKAGERELEASE']._serialized_start=1634
  _globals['_PACKAGERELEASE']._serialized_end=1689
  _globals['_PACKAGEYANK']._serialized_start=1691
  _globals['_PACKAGEYANK']._serialized_end=1721
# @@protoc_insertion_point(module_scope)



# coding: utf-8

"""
    Warg Registry API

    [warg](https://warg.io/) is an open source protocol for WebAssembly component registries. 

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic import BaseModel, Field, conint, constr, validator

class FetchLogsRequest(BaseModel):
    """
    A request to fetch logs from the registry.  # noqa: E501
    """
    log_length: conint(strict=True, ge=1) = Field(default=..., alias="logLength", description="The registry checkpoint log length to fetch from.")
    limit: Optional[conint(strict=True, le=1000, ge=1)] = Field(default=100, description="The limit of operator and packages records to return for the fetch request.")
    operator: Optional[constr(strict=True)] = Field(default=None, description="Represents a supported hash.")
    packages: Optional[Dict[str, Any]] = Field(default=None, description="The map of package log identifier to last known package record fetch token.  If the last package record identifier is null, records are returned from the start of the log. ")
    additional_properties: Dict[str, Any] = {}
    __properties = ["logLength", "limit", "operator", "packages"]

    @validator('operator')
    def operator_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[a-z0-9-]+:[a-f0-9]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-z0-9-]+:[a-f0-9]+$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> FetchLogsRequest:
        """Create an instance of FetchLogsRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> FetchLogsRequest:
        """Create an instance of FetchLogsRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return FetchLogsRequest.parse_obj(obj)

        _obj = FetchLogsRequest.parse_obj({
            "log_length": obj.get("logLength"),
            "limit": obj.get("limit") if obj.get("limit") is not None else 100,
            "operator": obj.get("operator"),
            "packages": obj.get("packages")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj





import os
import hashlib
import base64

import requests

from . import warg_proto
from .warg_client import WargClient
from .util import detect_registry_settings
from .model import Action, RegistryType, PackageDownload


def error(text):
    return ValueError(text)


def warg_pull(registry, warg_url, namespace, name, version=None, warg_token=None) -> PackageDownload:

    client = WargClient(
        registry=registry,
        warg_url=warg_url,
        warg_token=warg_token,
    )

    res = client.get_checkpoint(namespace=namespace)

    res = client.fetch_logs(
        namespace=namespace, name=name, log_length=res.contents.log_length)

    if not res.packages:
        raise error('failed to fetch logs')

    log_id, packages = res.packages.popitem()

    if not packages:
        raise error('failed to fetch logs')

    found_version, digest = find_version(packages, requested_version=version)

    if not digest or not found_version:
        raise error(
            'no package version found' if not version
            else 'requested version {} not found'.format(version))

    # get content sources
    res = client.get_content_sources(
        namespace=namespace,
        digest=digest,
    )

    if not res.content_sources:
        raise error('failed to fetch sources')

    digest, sources = res.content_sources.popitem()
    if not sources:
        raise error('failed to fetch sources')
    source = sources[0]
    url = source['url']

    # Fetch content from url with digest expected
    res = requests.get(url)
    content_digest = 'sha256:{}'.format(hashlib.sha256(res.content).hexdigest())
    if digest != content_digest:
        raise error('unexpected content digest')

    return PackageDownload(
        namespace=namespace,
        name=name,
        version=found_version,
        content=res.content,
        digest=digest,
    )


def find_version(packages, requested_version=None):
    version, digest = None, None
    for package in packages:
        record = warg_proto.PackageRecord()
        record.ParseFromString(base64.b64decode(package['contentBytes']))
        for entry in record.entries:

            obj = getattr(entry, entry.WhichOneof('contents'))

            if isinstance(obj, warg_proto.PackageRelease):

                if requested_version:
                    if obj.version == requested_version:
                        version = obj.version
                        digest = obj.content_hash
                else:
                    # latest version, assuming already ordered
                    version = obj.version
                    digest = obj.content_hash

    return version, digest




import os
import re
import sys
import requests
import semver

from .model import RegistryType


def add_github_output(key, value):
    """Add a github job output to the file pointed by the GITHUB_OUTPUT variable"""
    line = '{key}={value}\n'.format(key=key, value=value)
    if 'GITHUB_OUTPUT' in os.environ:
        with open(os.environ['GITHUB_OUTPUT'], 'a') as f:
            f.write(line)
    sys.stdout.write(line)

def detect_registry_settings(registry):
    """Discovery based on .well-known domain config"""
    url = "https://{domain}/.well-known/wasm-pkg/registry.json".format(domain=registry)
    r = requests.get(url, timeout=10)
    if not r.ok:
        return {}

    data = r.json()

    result = {}
    if 'preferredProtocol' in data:
        if data['preferredProtocol'] == 'warg':
            result['registry-type'] = RegistryType.WARG
            if 'warg' in data:
                if 'url' in data['warg']:
                    result['warg-url'] = data['warg']['url']

        elif data['preferredProtocol'] == 'oci':
            result['registry-type'] = RegistryType.OCI
            if 'oci' in data:
                if 'registry' in data['oci']:
                    result['oci-registry'] = data['oci']['registry']
                if 'namespacePrefix' in data['oci']:
                    result['oci-namespace-prefix'] = data['oci']['namespacePrefix']

    elif 'wargUrl' in data:
        result['registry-type'] = RegistryType.WARG
        result['warg-url'] = data['wargUrl']

    elif 'ociRegistry' in data:
        result['registry-type'] = RegistryType.OCI
        result['oci-registry'] = data['ociRegistry']
        if 'ociNamespacePrefix' in data:
            result['oci-namespace-prefix'] = data['ociNamespacePrefix']

    return result


def extract_version(filename):
    """Extract version from a filename.
    Return the substring between the last '@' and last '.'"""
    s = os.path.basename(filename)
    return s[s.rfind('@')+1:s.rfind('.')]


def format_package(namespace, name, version):
    """Canonical package representation in namespace:name@version format"""
    if version:
        return "{}:{}@{}".format(namespace, name, version)
    else:
        return "{}:{}".format(namespace, name)


def parse_package(package):
    """Parse a package string.

    The following forms are supported:
        namespace:name
        namespace:name@version
        namespace/name
        namespace/name@version

    Returns a tuple (namespace, name, version)
    """
    left, version = package, None
    if '@' in package:
        left, version = package.split('@', 1)
    if ':' in left:
        namespace, name = left.split(':', 1)
    elif '/' in left:
        namespace, name = left.split('/', 1)
    else:
        namespace, name = None, left

    if not namespace:
        raise ValueError("package namespace is required")
    if not name:
        raise ValueError("package version is required")

    if not re.match(r"[\w-]+", namespace):
        raise ValueError("invalid package namespace")
    if not re.match(r"[\w-]+", name):
        raise ValueError("invalid package name")
    if version:
        # throws on failed validation
        semver.Version.parse(version)
    return namespace, name, version




from urllib.parse import urlparse
import hashlib
import urllib3

import leb128
import warg_openapi as warg


class WargClient:

    def __init__(self, registry, warg_url, warg_token=None, stealth=False):
        self.registry = registry

        configuration = warg.Configuration(
            host = "{}{}".format(warg_url, 'v1' if warg_url.endswith('/') else '/v1'),
        )

        # Support for digest strings as path params: sha256:<digest>
        # or else they get quoted, resulting in "Invalid content digest" error.
        configuration.safe_chars_for_path_param = "/:"

        client = warg.ApiClient(
            configuration=configuration,
        )

        # private packages
        if warg_token:
            client.default_headers['Authorization'] = 'Bearer {}'.format(warg_token)

        if stealth:
            client.default_headers['User-Agent'] = urllib3.util.SKIP_HEADER

        self.fetch_api = warg.FetchApi(client)
        self.content_api = warg.ContentApi(client)

    def get_warg_registry(self, namespace):
        return "{}.{}".format(namespace, self.registry)

    def get_checkpoint(self, namespace):
        return self.fetch_api.get_checkpoint(warg_registry=self.get_warg_registry(namespace))

    def fetch_logs(self, namespace, name, log_length):
        log_id = generate_log_id(namespace, name)
        return self.fetch_api.fetch_logs(
            fetch_logs_request=warg.FetchLogsRequest(
                log_length=log_length,
                packages={
                    log_id: None,
                },
            ),
            warg_registry=self.get_warg_registry(namespace),
        )

    def fetch_names(self, namespace, name):
        """Translate log_id to package name."""
        log_id = generate_log_id(namespace, name)
        request = warg.FetchPackageNamesRequest(packages=[log_id])
        return self.fetch_api.fetch_names(
            fetch_package_names_request=request,
            warg_registry=self.get_warg_registry(namespace))

    def get_content_sources(self, namespace, digest):
        return self.content_api.get_content_sources(
            digest=digest,
            warg_registry=self.get_warg_registry(namespace)
        )


def generate_log_id(namespace, name):
    """Derive logId from package name"""
    package_name = "{}:{}".format(namespace, name)
    s = hashlib.sha256()
    s.update(b'WARG-PACKAGE-LOG-ID-V0:')
    s.update(b'WARG-PACKAGE-ID-V0')
    s.update(leb128.u.encode(len(package_name)))
    s.update(package_name.encode('utf8'))
    return 'sha256:{}'.format(s.hexdigest())



# coding: utf-8

"""
    Warg Registry API

    [warg](https://warg.io/) is an open source protocol for WebAssembly component registries. 

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic import BaseModel, Field, StrictBool, conlist
from warg_openapi.models.fetch_warning import FetchWarning
from warg_openapi.models.published_record_envelope import PublishedRecordEnvelope

class FetchLogsResponse(BaseModel):
    """
    A response containing the requested logs.  # noqa: E501
    """
    more: Optional[StrictBool] = Field(default=None, description="Whether there may be more records available.  This occurs when the number of records returned for a log reaches the requested limit.  If `true`, the client should make another request with the new last known record identifiers. ")
    operator: Optional[conlist(PublishedRecordEnvelope, max_items=1000)] = Field(default=None, description="The operator log records for the given checkpoint since the last known record.")
    warnings: Optional[conlist(FetchWarning, max_items=1000)] = Field(default=None, description="An optional list of warnings.")
    packages: Optional[Dict[str, Any]] = Field(default=None, description="The map of package log identifier to package records.")
    __properties = ["more", "operator", "warnings", "packages"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> FetchLogsResponse:
        """Create an instance of FetchLogsResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in operator (list)
        _items = []
        if self.operator:
            for _item in self.operator:
                if _item:
                    _items.append(_item.to_dict())
            _dict['operator'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in warnings (list)
        _items = []
        if self.warnings:
            for _item in self.warnings:
                if _item:
                    _items.append(_item.to_dict())
            _dict['warnings'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> FetchLogsResponse:
        """Create an instance of FetchLogsResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return FetchLogsResponse.parse_obj(obj)

        _obj = FetchLogsResponse.parse_obj({
            "more": obj.get("more"),
            "operator": [PublishedRecordEnvelope.from_dict(_item) for _item in obj.get("operator")] if obj.get("operator") is not None else None,
            "warnings": [FetchWarning.from_dict(_item) for _item in obj.get("warnings")] if obj.get("warnings") is not None else None,
            "packages": obj.get("packages")
        })
        return _obj





# coding: utf-8

"""
    Warg Registry API

    [warg](https://warg.io/) is an open source protocol for WebAssembly component registries. 

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr, validator

class CheckpointVerificationResponse(BaseModel):
    """
    CheckpointVerificationResponse
    """
    checkpoint: StrictStr = Field(default=..., description="The verification of the checkpoint's `logLength`, `logRoot` and `mapRoot`:    * `unverified`: checkpoint may be valid or invalid; if `retryAfter` is provided, the client should retry the request;   * `verified`: `logLength`, `logRoot` and `mapRoot` are verified;   * `invalid`: checkpoint with the specified `logLength` was either not produced or does not match the correct `logRoot` and `mapRoot`; ")
    signature: StrictStr = Field(default=..., description="The verification of the checkpoint's `keyId` and `signature`:    * `unverified`: checkpoint's `signature` may be valid or invalid; if `retryAfter` is provided, the client should retry the request;   * `verified`: checkpoint's `signature` is verified;   * `invalid`: `keyId` is not known or does not have authorization (could have been revoke or never granted) to sign checkpoints or the `signature` itself is invalid; ")
    retry_after: Optional[StrictInt] = Field(default=None, alias="retryAfter", description="If either `checkpoint` or `signature` is `unverified` status, then the server may instruct the client to retry the request after the specified number of seconds.")
    __properties = ["checkpoint", "signature", "retryAfter"]

    @validator('checkpoint')
    def checkpoint_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('unverified', 'verified', 'invalid',):
            raise ValueError("must be one of enum values ('unverified', 'verified', 'invalid')")
        return value

    @validator('signature')
    def signature_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('unverified', 'verified', 'invalid',):
            raise ValueError("must be one of enum values ('unverified', 'verified', 'invalid')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CheckpointVerificationResponse:
        """Create an instance of CheckpointVerificationResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CheckpointVerificationResponse:
        """Create an instance of CheckpointVerificationResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CheckpointVerificationResponse.parse_obj(obj)

        _obj = CheckpointVerificationResponse.parse_obj({
            "checkpoint": obj.get("checkpoint"),
            "signature": obj.get("signature"),
            "retry_after": obj.get("retryAfter")
        })
        return _obj





# coding: utf-8

# flake8: noqa

"""
    Warg Registry API

    [warg](https://warg.io/) is an open source protocol for WebAssembly component registries. 

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# Define package exports
__all__ = [
    "ContentApi",
    "FetchApi",
    "LedgerApi",
    "MonitorApi",
    "PackageApi",
    "ProofApi",
    "ApiResponse",
    "ApiClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
    "BundleFailureError",
    "Checkpoint",
    "CheckpointVerificationResponse",
    "ContentSourcesResponse",
    "EnvelopeBody",
    "Error",
    "FetchLogs404Response",
    "FetchLogsIDNotFoundError",
    "FetchLogsLogLengthNotFoundError",
    "FetchLogsRequest",
    "FetchLogsResponse",
    "FetchNames404Response",
    "FetchPackageNamesRequest",
    "FetchPackageNamesResponse",
    "FetchWarning",
    "GetContentSources404Response",
    "GetPackageRecord404Response",
    "HttpGet",
    "HttpUpload",
    "HttpUploadHeaders",
    "IncorrectProofError",
    "LedgerSourcesResponse",
    "LedgerSourcesResponseSourcesInner",
    "MissingContent",
    "PackageNotIncludedError",
    "PackageRecord",
    "ProcessingRecord",
    "ProveConsistency404Response",
    "ProveConsistencyRequest",
    "ProveConsistencyResponse",
    "ProveInclusion404Response",
    "ProveInclusion422Response",
    "ProveInclusionRequest",
    "ProveInclusionResponse",
    "PublishPackageRecord404Response",
    "PublishPackageRecord409Response",
    "PublishPackageRecordRequest",
    "PublishedRecord",
    "PublishedRecordEnvelope",
    "RejectedRecord",
    "Signature",
    "SignedCheckpoint",
    "SourcingRecord",
    "TimestampedCheckpoint",
]

# import apis into sdk package
from warg_openapi.api.content_api import ContentApi as ContentApi
from warg_openapi.api.fetch_api import FetchApi as FetchApi
from warg_openapi.api.ledger_api import LedgerApi as LedgerApi
from warg_openapi.api.monitor_api import MonitorApi as MonitorApi
from warg_openapi.api.package_api import PackageApi as PackageApi
from warg_openapi.api.proof_api import ProofApi as ProofApi

# import ApiClient
from warg_openapi.api_response import ApiResponse as ApiResponse
from warg_openapi.api_client import ApiClient as ApiClient
from warg_openapi.configuration import Configuration as Configuration
from warg_openapi.exceptions import OpenApiException as OpenApiException
from warg_openapi.exceptions import ApiTypeError as ApiTypeError
from warg_openapi.exceptions import ApiValueError as ApiValueError
from warg_openapi.exceptions import ApiKeyError as ApiKeyError
from warg_openapi.exceptions import ApiAttributeError as ApiAttributeError
from warg_openapi.exceptions import ApiException as ApiException

# import models into sdk package
from warg_openapi.models.bundle_failure_error import BundleFailureError as BundleFailureError
from warg_openapi.models.checkpoint import Checkpoint as Checkpoint
from warg_openapi.models.checkpoint_verification_response import CheckpointVerificationResponse as CheckpointVerificationResponse
from warg_openapi.models.content_sources_response import ContentSourcesResponse as ContentSourcesResponse
from warg_openapi.models.envelope_body import EnvelopeBody as EnvelopeBody
from warg_openapi.models.error import Error as Error
from warg_openapi.models.fetch_logs404_response import FetchLogs404Response as FetchLogs404Response
from warg_openapi.models.fetch_logs_id_not_found_error import FetchLogsIDNotFoundError as FetchLogsIDNotFoundError
from warg_openapi.models.fetch_logs_log_length_not_found_error import FetchLogsLogLengthNotFoundError as FetchLogsLogLengthNotFoundError
from warg_openapi.models.fetch_logs_request import FetchLogsRequest as FetchLogsRequest
from warg_openapi.models.fetch_logs_response import FetchLogsResponse as FetchLogsResponse
from warg_openapi.models.fetch_names404_response import FetchNames404Response as FetchNames404Response
from warg_openapi.models.fetch_package_names_request import FetchPackageNamesRequest as FetchPackageNamesRequest
from warg_openapi.models.fetch_package_names_response import FetchPackageNamesResponse as FetchPackageNamesResponse
from warg_openapi.models.fetch_warning import FetchWarning as FetchWarning
from warg_openapi.models.get_content_sources404_response import GetContentSources404Response as GetContentSources404Response
from warg_openapi.models.get_package_record404_response import GetPackageRecord404Response as GetPackageRecord404Response
from warg_openapi.models.http_get import HttpGet as HttpGet
from warg_openapi.models.http_upload import HttpUpload as HttpUpload
from warg_openapi.models.http_upload_headers import HttpUploadHeaders as HttpUploadHeaders
from warg_openapi.models.incorrect_proof_error import IncorrectProofError as IncorrectProofError
from warg_openapi.models.ledger_sources_response import LedgerSourcesResponse as LedgerSourcesResponse
from warg_openapi.models.ledger_sources_response_sources_inner import LedgerSourcesResponseSourcesInner as LedgerSourcesResponseSourcesInner
from warg_openapi.models.missing_content import MissingContent as MissingContent
from warg_openapi.models.package_not_included_error import PackageNotIncludedError as PackageNotIncludedError
from warg_openapi.models.package_record import PackageRecord as PackageRecord
from warg_openapi.models.processing_record import ProcessingRecord as ProcessingRecord
from warg_openapi.models.prove_consistency404_response import ProveConsistency404Response as ProveConsistency404Response
from warg_openapi.models.prove_consistency_request import ProveConsistencyRequest as ProveConsistencyRequest
from warg_openapi.models.prove_consistency_response import ProveConsistencyResponse as ProveConsistencyResponse
from warg_openapi.models.prove_inclusion404_response import ProveInclusion404Response as ProveInclusion404Response
from warg_openapi.models.prove_inclusion422_response import ProveInclusion422Response as ProveInclusion422Response
from warg_openapi.models.prove_inclusion_request import ProveInclusionRequest as ProveInclusionRequest
from warg_openapi.models.prove_inclusion_response import ProveInclusionResponse as ProveInclusionResponse
from warg_openapi.models.publish_package_record404_response import PublishPackageRecord404Response as PublishPackageRecord404Response
from warg_openapi.models.publish_package_record409_response import PublishPackageRecord409Response as PublishPackageRecord409Response
from warg_openapi.models.publish_package_record_request import PublishPackageRecordRequest as PublishPackageRecordRequest
from warg_openapi.models.published_record import PublishedRecord as PublishedRecord
from warg_openapi.models.published_record_envelope import PublishedRecordEnvelope as PublishedRecordEnvelope
from warg_openapi.models.rejected_record import RejectedRecord as RejectedRecord
from warg_openapi.models.signature import Signature as Signature
from warg_openapi.models.signed_checkpoint import SignedCheckpoint as SignedCheckpoint
from warg_openapi.models.sourcing_record import SourcingRecord as SourcingRecord
from warg_openapi.models.timestamped_checkpoint import TimestampedCheckpoint as TimestampedCheckpoint



# coding: utf-8

"""
    Warg Registry API

    [warg](https://warg.io/) is an open source protocol for WebAssembly component registries. 

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic import BaseModel, Field, conint, constr, validator

class Checkpoint(BaseModel):
    """
    A registry checkpoint.  Checkpoints are hashed by concatenating the following:   * A prefix of the byte string `WARG-CHECKPOINT-V0`   * The LEB128-encoded `logLength`   * The LEB128-length-prefixed `logRoot`   * The LEB128-length-prefixed `mapRoot`   # noqa: E501
    """
    log_length: conint(strict=True, ge=1) = Field(default=..., alias="logLength", description="The log length of the checkpoint.")
    log_root: constr(strict=True) = Field(default=..., alias="logRoot", description="Represents a supported hash.")
    map_root: constr(strict=True) = Field(default=..., alias="mapRoot", description="Represents a supported hash.")
    __properties = ["logLength", "logRoot", "mapRoot"]

    @validator('log_root')
    def log_root_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-z0-9-]+:[a-f0-9]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-z0-9-]+:[a-f0-9]+$/")
        return value

    @validator('map_root')
    def map_root_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-z0-9-]+:[a-f0-9]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-z0-9-]+:[a-f0-9]+$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Checkpoint:
        """Create an instance of Checkpoint from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Checkpoint:
        """Create an instance of Checkpoint from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Checkpoint.parse_obj(obj)

        _obj = Checkpoint.parse_obj({
            "log_length": obj.get("logLength"),
            "log_root": obj.get("logRoot"),
            "map_root": obj.get("mapRoot")
        })
        return _obj





# coding: utf-8

"""
    Warg Registry API

    [warg](https://warg.io/) is an open source protocol for WebAssembly component registries. 

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic import BaseModel, Field, constr, validator
from warg_openapi.models.timestamped_checkpoint import TimestampedCheckpoint

class SignedCheckpoint(BaseModel):
    """
    A signed registry checkpoint.  # noqa: E501
    """
    key_id: constr(strict=True) = Field(default=..., alias="keyId", description="Represents a supported hash.")
    signature: constr(strict=True) = Field(default=..., description="The algorithm-prefixed bytes of the signature (base64 encoded).")
    contents: TimestampedCheckpoint = Field(...)
    __properties = ["keyId", "signature", "contents"]

    @validator('key_id')
    def key_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-z0-9-]+:[a-f0-9]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-z0-9-]+:[a-f0-9]+$/")
        return value

    @validator('signature')
    def signature_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-z0-9-]+:(?:[A-Za-z0-9+\/]{4})*(?:[A-Za-z0-9+\/]{4}|[A-Za-z0-9+\/]{3}=|[A-Za-z0-9+\/]{2}={2})$", value):
            raise ValueError(r"must validate the regular expression /^[a-z0-9-]+:(?:[A-Za-z0-9+\/]{4})*(?:[A-Za-z0-9+\/]{4}|[A-Za-z0-9+\/]{3}=|[A-Za-z0-9+\/]{2}={2})$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SignedCheckpoint:
        """Create an instance of SignedCheckpoint from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of contents
        if self.contents:
            _dict['contents'] = self.contents.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SignedCheckpoint:
        """Create an instance of SignedCheckpoint from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SignedCheckpoint.parse_obj(obj)

        _obj = SignedCheckpoint.parse_obj({
            "key_id": obj.get("keyId"),
            "signature": obj.get("signature"),
            "contents": TimestampedCheckpoint.from_dict(obj.get("contents")) if obj.get("contents") is not None else None
        })
        return _obj





import sys
import enum
import glob
import os
import hashlib
import base64

import click
import semver
import validators
import requests

from . import warg_proto
from .warg_crypto import PrivateKey
from .warg_client import WargClient
from .util import add_github_output, detect_registry_settings, RegistryType
from .util import format_package, parse_package
from .model import Action, RegistryType
from .warg_pull import warg_pull


def error(text):
    return ValueError(text)

@click.group()
def cli():
    pass


@cli.command(help="Push to a WebAssembly registry")
@click.option('--registry', required=True, help="registry domain name")
@click.option('--namespace', required=True, help="package namespace")
@click.option('--name', required=True, help="package name")
@click.option('--version', required=True, help="package version")
@click.option('--path', required=True, help="filename")
def push(registry, namespace, name, version, path):

    if not path:
        raise error('path is required')

    # expand path pattern into filename, ensure only a single file
    files = glob.glob(path)
    if not files:
        raise error('file not found: {}'.format(path))
    if len(files) > 1:
        raise error('more than one files found: {}'.format(path))

    filename = files[0]

    # extract version from filename if not provided
    if not version:
        version = extract_version(filename)

    # validate version as semver
    semver.Version.parse(version)


    # validate key
    key = os.environ.get('WARG_PRIVATE_KEY')
    if not key:
        raise error('no key provided')

    try:
        PrivateKey.load(key)
    except:
        raise error("Error loading private key")
    else:
        print("valid private key")

    # todo: implement push


@cli.command(help="Pull from a WebAssembly registry")
@click.option('--registry', required=True, help="registry domain name")
@click.option('--package', required=True, help="package spec")
@click.option('--path', required=False, help="filename")
@click.option('--warg-token', required=False, envvar='WARG_TOKEN', help="warg token")
def pull(registry, package, path=None, warg_token=None):

    if not package:
        raise error("package is required")

    namespace, name, version = parse_package(package)

    settings = validate_registry(registry)

    if settings.get('registry-type') != RegistryType.WARG:
        raise error("Registry type not supported: {}".format(settings.get('registry-type')))

    download = warg_pull(registry, settings['warg-url'], namespace, name, version, warg_token=warg_token)

    filename = path or "{}:{}@{}.wasm".format(namespace, name, download.version)
    with open(filename, 'wb') as f:
        f.write(download.content)

    add_github_output('package', format_package(namespace=namespace, name=name, version=download.version))
    add_github_output('package-namespace', download.namespace)
    add_github_output('package-name', download.name)
    add_github_output('package-version', download.version)
    add_github_output('digest', download.digest)
    add_github_output('filename', filename)


def validate_registry(registry):
    if not registry:
        raise error('registry is required')

    if not validators.domain(registry, consider_tld=True):
        raise error('registry is not a valid domain name: "{}"'.format(registry))

    add_github_output('registry', registry)
    settings = detect_registry_settings(registry)
    for key, value in settings.items():
        add_github_output(key, str(value))
    return settings


def main():
    cli()


if __name__ == "__main__":
    main()



# coding: utf-8

"""
    Warg Registry API

    [warg](https://warg.io/) is an open source protocol for WebAssembly component registries. 

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic import BaseModel, Field, constr
from warg_openapi.models.envelope_body import EnvelopeBody

class PublishPackageRecordRequest(BaseModel):
    """
    A request to publish a record to a package log.  # noqa: E501
    """
    package_name: constr(strict=True, max_length=128) = Field(default=..., alias="packageName", description="The name of the package log being published to.")
    record: EnvelopeBody = Field(default=..., description="The package record being published to the log.")
    content_sources: Optional[Dict[str, Any]] = Field(default=None, alias="contentSources", description="The map of content digest to sources.")
    __properties = ["packageName", "record", "contentSources"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PublishPackageRecordRequest:
        """Create an instance of PublishPackageRecordRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of record
        if self.record:
            _dict['record'] = self.record.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PublishPackageRecordRequest:
        """Create an instance of PublishPackageRecordRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PublishPackageRecordRequest.parse_obj(obj)

        _obj = PublishPackageRecordRequest.parse_obj({
            "package_name": obj.get("packageName"),
            "record": EnvelopeBody.from_dict(obj.get("record")) if obj.get("record") is not None else None,
            "content_sources": obj.get("contentSources")
        })
        return _obj





# flake8: noqa

# import apis into api package
from warg_openapi.api.content_api import ContentApi
from warg_openapi.api.fetch_api import FetchApi
from warg_openapi.api.ledger_api import LedgerApi
from warg_openapi.api.monitor_api import MonitorApi
from warg_openapi.api.package_api import PackageApi
from warg_openapi.api.proof_api import ProofApi





import enum
from collections import namedtuple


class Action(enum.Enum):
    PUSH = 'push'
    PULL = 'pull'

    def __str__(self):
        return self.value


class RegistryType(enum.Enum):
    OCI = 'oci'
    WARG = 'warg'

    def __str__(self):
        return self.value


PackageDownload = namedtuple('PackageDownload', (
    'namespace',
    'name',
    'version',
    'content',
    'digest',
    ))



# coding: utf-8

"""
    Warg Registry API

    [warg](https://warg.io/) is an open source protocol for WebAssembly component registries. 

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr, validator
from warg_openapi.models.http_upload_headers import HttpUploadHeaders

class HttpUpload(BaseModel):
    """
    A HTTP upload endpoint.  # noqa: E501
    """
    type: StrictStr = Field(default=..., description="The type of upload endpoint.")
    method: StrictStr = Field(default=..., description="The HTTP method for the upload request. Uppercase is required.")
    url: StrictStr = Field(default=..., description="The URL of the upload endpoint, which may be relative to the API base URL.")
    headers: Optional[HttpUploadHeaders] = None
    __properties = ["type", "method", "url", "headers"]

    @validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('http',):
            raise ValueError("must be one of enum values ('http')")
        return value

    @validator('method')
    def method_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('POST', 'PUT',):
            raise ValueError("must be one of enum values ('POST', 'PUT')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> HttpUpload:
        """Create an instance of HttpUpload from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of headers
        if self.headers:
            _dict['headers'] = self.headers.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> HttpUpload:
        """Create an instance of HttpUpload from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return HttpUpload.parse_obj(obj)

        _obj = HttpUpload.parse_obj({
            "type": obj.get("type"),
            "method": obj.get("method"),
            "url": obj.get("url"),
            "headers": HttpUploadHeaders.from_dict(obj.get("headers")) if obj.get("headers") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Warg Registry API

    [warg](https://warg.io/) is an open source protocol for WebAssembly component registries. 

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic import BaseModel, Field, StrictInt, StrictStr, validator

class ProveInclusion404Response(BaseModel):
    """
    ProveInclusion404Response
    """
    status: StrictInt = Field(default=..., description="The HTTP status code for the error.")
    type: StrictStr = Field(default=..., description="The type of entity that was not found.")
    id: StrictInt = Field(default=..., description="The identifier of the entity that was not found.")
    additional_properties: Dict[str, Any] = {}
    __properties = ["status", "type", "id"]

    @validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('logLength', 'leaf',):
            raise ValueError("must be one of enum values ('logLength', 'leaf')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ProveInclusion404Response:
        """Create an instance of ProveInclusion404Response from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ProveInclusion404Response:
        """Create an instance of ProveInclusion404Response from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ProveInclusion404Response.parse_obj(obj)

        _obj = ProveInclusion404Response.parse_obj({
            "status": obj.get("status"),
            "type": obj.get("type"),
            "id": obj.get("id")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj





# coding: utf-8

"""
    Warg Registry API

    [warg](https://warg.io/) is an open source protocol for WebAssembly component registries. 

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr, validator

class LedgerSourcesResponseSourcesInner(BaseModel):
    """
    Ledger source HTTP get URL.  # noqa: E501
    """
    first_registry_index: StrictInt = Field(default=..., alias="firstRegistryIndex", description="The first registry index included in the source.")
    last_registry_index: StrictInt = Field(default=..., alias="lastRegistryIndex", description="The last registry index included in the source.")
    url: StrictStr = Field(default=..., description="The HTTP GET URL to fetch the source.")
    content_type: StrictStr = Field(default=..., alias="contentType", description="The content type of source.")
    accept_ranges: Optional[StrictBool] = Field(default=None, alias="acceptRanges", description="Flag indicating if the server accepts byte ranges with `Range` header.")
    __properties = ["firstRegistryIndex", "lastRegistryIndex", "url", "contentType", "acceptRanges"]

    @validator('content_type')
    def content_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('application/vnd.warg.ledger.packed',):
            raise ValueError("must be one of enum values ('application/vnd.warg.ledger.packed')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> LedgerSourcesResponseSourcesInner:
        """Create an instance of LedgerSourcesResponseSourcesInner from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> LedgerSourcesResponseSourcesInner:
        """Create an instance of LedgerSourcesResponseSourcesInner from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return LedgerSourcesResponseSourcesInner.parse_obj(obj)

        _obj = LedgerSourcesResponseSourcesInner.parse_obj({
            "first_registry_index": obj.get("firstRegistryIndex"),
            "last_registry_index": obj.get("lastRegistryIndex"),
            "url": obj.get("url"),
            "content_type": obj.get("contentType"),
            "accept_ranges": obj.get("acceptRanges")
        })
        return _obj


